@@ -18,7 +18,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93 (BSDI)";
  * Start Date:  Fri, 17 Jan 1997 16:09:10 +0100
  * Project:     INN (innfeed)
  * File:        config.y
- * RCSId:       $Id: configfile.y,v 1.3 1997/02/14 02:17:04 brister Exp $
+ * RCSId:       $Id: configfile.y,v 1.5 1997/03/03 12:06:02 brister Exp $
  * Description: 
  * 
  */
@@ -139,6 +139,7 @@ void freeScopeTree (scope *s)
 	freeValue (s->values [i]) ;
 
   free (s->values) ;
+  free (s->scope_type) ;
 
   s->parent = NULL ;
   s->values = NULL ;
@@ -219,7 +220,7 @@ char *addReal (scope *s, const char *name, double val)
   return NULL ;
 }
 
-char *addString (scope *s, const char *name, char *val)
+char *addString (scope *s, const char *name, const char *val)
 {
   value *v ;
   char *error ;
@@ -230,7 +231,7 @@ char *addString (scope *s, const char *name, char *val)
   v = (value *) calloc (1,sizeof (value)) ;
   v->name = strdup (name) ;
   v->type = stringval ;
-  v->v.charp_val = val ;
+  v->v.charp_val = strdup (val) ;
   
   addValue (s,v) ;
 
@@ -368,6 +369,7 @@ static void freeValue (value *v)
       default:
         break ;
     }
+  free (v) ;
 }
 
 static char *checkName (scope *s, const char *name)
@@ -679,7 +681,7 @@ static int doCallbacks (void)
 
 
 static char *key ;
-#line 671 "configfile.y"
+#line 673 "configfile.y"
 typedef union{
     scope *scp ;
     value *val ;
@@ -689,7 +691,7 @@ typedef union{
     char *string ;
     char chr ;
 } YYSTYPE;
-#line 693 "config_y.c"
+#line 695 "y.tab.c"
 #define PEER 257
 #define GROUP 258
 #define IVAL 259
@@ -700,8 +702,8 @@ typedef union{
 #define COLON 264
 #define LBRACE 265
 #define RBRACE 266
-#define TRUE 267
-#define FALSE 268
+#define TRUEBVAL 267
+#define FALSEBVAL 268
 #define CHAR 269
 #define WORD 270
 #define IP_ADDRESS 271
@@ -763,8 +765,8 @@ char *yyname[] = {
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"PEER","GROUP","IVAL","RVAL",
-"NAME","STRING","SCOPE","COLON","LBRACE","RBRACE","TRUE","FALSE","CHAR","WORD",
-"IP_ADDRESS",
+"NAME","STRING","SCOPE","COLON","LBRACE","RBRACE","TRUEBVAL","FALSEBVAL","CHAR",
+"WORD","IP_ADDRESS",
 };
 char *yyrule[] = {
 "$accept : input",
@@ -783,8 +785,8 @@ char *yyrule[] = {
 "entry : WORD $$4 COLON value",
 "value : WORD",
 "value : IVAL",
-"value : TRUE",
-"value : FALSE",
+"value : TRUEBVAL",
+"value : FALSEBVAL",
 "value : RVAL",
 "value : STRING",
 "value : CHAR",
@@ -815,7 +817,7 @@ struct yystack {
 int yychar; /* some people use this, so we copy it in & out */
 int yyerrflag; /* must be global for yyerrok & YYRECOVERING */
 YYSTYPE yylval;
-#line 771 "configfile.y"
+#line 787 "configfile.y"
 
 int yyerror (const char *s)
 {
@@ -833,6 +835,14 @@ int yywrap (void)
   return 1 ;
 }
 
+extern FILE *yyin ;
+extern int yydebug ;
+
+#define NO_INHERIT 0
+
+
+#if ! defined (WANT_MAIN)
+
 struct peer_table_s
 {
     char *peerName ;
@@ -843,8 +853,19 @@ static struct peer_table_s *peerTable ;
 static int peerTableCount ;
 static int peerTableIdx ;
 
-#define NO_INHERIT 0
+void configCleanup (void)
+{
+  int i ;
 
+  for (i = 0 ; i < peerTableIdx ; i++)
+    free (peerTable[i].peerName) ;
+  free (peerTable) ;
+  
+  freeScopeTree (topScope);
+  free (funcs) ;
+  free (args) ;
+}
+  
 
 int buildPeerTable (FILE *fp, scope *s)
 {
@@ -859,7 +880,7 @@ int buildPeerTable (FILE *fp, scope *s)
             {
               if (strcmp (peerTable[j].peerName,s->values[i]->name) == 0)
                 {
-                  logOrPrint (LOG_ERR,fp,DUPPEERNAME,peerTable[j].peerName) ;
+                  logOrPrint (LOG_ERR,fp,DUP_PEER_NAME,peerTable[j].peerName) ;
                   rval = 0 ;
                   break ;
                 }
@@ -893,17 +914,20 @@ int buildPeerTable (FILE *fp, scope *s)
 /* read the config file. Any errors go to errorDest if it is non-NULL,
    otherwise they are syslogged. If justCheck is true then return after
    parsing */
-
-extern FILE *yyin ;
-extern int yydebug ;
-
+static int inited = 0 ;
 int readConfig (const char *file, FILE *errorDest, int justCheck, int dump)
 {
   scope *oldTop = topScope ;
   FILE *fp ;
   int rval ;
 
-  yydebug = (getenv ("YYDEBUG") == NULL ? 0 : 1) ;
+  if (!inited)
+    {
+      inited = 1 ;
+      yydebug = (getenv ("YYDEBUG") == NULL ? 0 : 1) ;
+      if (yydebug)
+        atexit (configCleanup) ;
+    }
 
   if (file == NULL || strlen (file) == NULL || !fileExistsP (file))
     {
@@ -966,12 +990,13 @@ int readConfig (const char *file, FILE *errorDest, int justCheck, int dump)
     }
   else
     {
-      for ( ; peerTableIdx > 0 ; peerTableIdx--)
+      for (peerTableIdx-- ; peerTableIdx >= 0 ; peerTableIdx--)
         {
           free (peerTable [peerTableIdx].peerName) ;
           peerTable [peerTableIdx].peerName = NULL ;
           peerTable [peerTableIdx].peerValue = NULL ;
         }
+      peerTableIdx = 0 ;
       
       if (!buildPeerTable (errorDest,topScope))
         logAndExit (1,"Failed to build list of peers") ;
@@ -998,7 +1023,7 @@ value *getNextPeer (int *cookie)
 
 value *findPeer (const char *name)
 {
-  value *v ;
+  value *v = NULL ;
   int i ;
 
   for (i = 0 ; i < peerTableIdx ; i++)
@@ -1011,7 +1036,7 @@ value *findPeer (const char *name)
   return v ;
 }
 
-
+#endif
 
 #if defined (WANT_MAIN)
 int main (int argc, char **argv) {
@@ -1085,7 +1110,7 @@ int main (int argc, char **argv) {
 #endif /* defined (WANT_MAIN) */
 
 
-#line 1089 "config_y.c"
+#line 1114 "y.tab.c"
 /* allocate initial stack */
 #if defined(__STDC__) || defined(__cplusplus)
 static int yyinitstack(struct yystack *sp)
@@ -1281,7 +1306,7 @@ yyreduce:
     switch (yyn)
     {
 case 1:
-#line 698 "configfile.y"
+#line 700 "configfile.y"
 { 	
 		lineCount = 1 ;
 		addScope (NULL,"",newScope ("")) ;
@@ -1289,11 +1314,11 @@ case 1:
 	}
 break;
 case 2:
-#line 702 "configfile.y"
+#line 704 "configfile.y"
 { if (!doCallbacks()) YYABORT ; }
 break;
 case 6:
-#line 708 "configfile.y"
+#line 710 "configfile.y"
 {
 		errbuff = malloc (strlen(SYNTAX_ERROR) + 12) ;
 		sprintf (errbuff,SYNTAX_ERROR,lineCount) ;
@@ -1301,42 +1326,46 @@ case 6:
 	}
 break;
 case 7:
-#line 715 "configfile.y"
+#line 717 "configfile.y"
 {
 		errbuff = addScope (currScope,yyvsp[-1].name,newScope ("peer")) ;
+                free (yyvsp[-1].name) ;
 		if (errbuff != NULL) YYABORT ;
 	}
 break;
 case 8:
-#line 718 "configfile.y"
+#line 721 "configfile.y"
 {
 		currScope = currScope->parent ;
 	}
 break;
 case 9:
-#line 721 "configfile.y"
+#line 724 "configfile.y"
 {
 		errbuff = addScope (currScope,yyvsp[-1].name,newScope ("group")) ;
+                free (yyvsp[-1].name) ;
 		if (errbuff != NULL) YYABORT ;
 	}
 break;
 case 10:
-#line 724 "configfile.y"
+#line 728 "configfile.y"
 {
 		currScope = currScope->parent ;
 	}
 break;
 case 11:
-#line 727 "configfile.y"
+#line 731 "configfile.y"
 {
 		errbuff = malloc (strlen(UNKNOWN_SCOPE_TYPE) + 15 +
 					  strlen (yyvsp[-2].name)) ;
 		sprintf (errbuff,UNKNOWN_SCOPE_TYPE,lineCount,yyvsp[-2].name) ;
+                free (yyvsp[-2].name) ;
+                free (yyvsp[-1].name) ;
 		YYABORT ;
 	}
 break;
 case 12:
-#line 733 "configfile.y"
+#line 739 "configfile.y"
 { 
 		if ((errbuff = keyOk(yyvsp[0].name)) != NULL) 
 			YYABORT ;
@@ -1345,55 +1374,65 @@ case 12:
 	}
 break;
 case 14:
-#line 740 "configfile.y"
+#line 746 "configfile.y"
 {
 		if ((errbuff = addString (currScope, key, yyvsp[0].name)) != NULL)
-			YYABORT ; 
+			YYABORT ;
+                free (key) ;
+                free (yyvsp[0].name) ;
 	}
 break;
 case 15:
-#line 744 "configfile.y"
+#line 752 "configfile.y"
 {
 		if ((errbuff = addInteger(currScope, key, yyvsp[0].integer)) != NULL)
 			YYABORT; 
+                free (key) ;
 	}
 break;
 case 16:
-#line 748 "configfile.y"
+#line 757 "configfile.y"
 {
 		if ((errbuff = addBoolean (currScope, key, 1)) != NULL)
 			YYABORT ; 
+                free (key) ;
+                free (yyvsp[0].name) ;
 	}
 break;
 case 17:
-#line 752 "configfile.y"
+#line 763 "configfile.y"
 {
 		if ((errbuff = addBoolean (currScope, key, 0)) != NULL)
 			YYABORT ; 
+                free (key) ;
+                free (yyvsp[0].name) ;
 	}
 break;
 case 18:
-#line 756 "configfile.y"
+#line 769 "configfile.y"
 {
 		if ((errbuff = addReal (currScope, key, yyvsp[0].real)) != NULL)
 			YYABORT ; 
+                free (key) ;
 	}
 break;
 case 19:
-#line 760 "configfile.y"
+#line 774 "configfile.y"
 { 
 		if ((errbuff = addString (currScope, key, yyvsp[0].string)) != NULL)
 			YYABORT;
+                free (key) ;
 	}
 break;
 case 20:
-#line 764 "configfile.y"
+#line 779 "configfile.y"
 {
 		if ((errbuff = addChar (currScope, key, yyvsp[0].chr)) != NULL)
 			YYABORT ;
+                free (key) ;
         }
 break;
-#line 1397 "config_y.c"
+#line 1436 "y.tab.c"
     }
     yystk.ssp -= yym;
     yystate = *yystk.ssp;
