@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 08:45:24 1995
  * Project:     INN (innfeed)
  * File:        connection.h
- * RCSId:       $Id: connection.h,v 1.10 1996/12/07 01:32:17 brister Exp $
+ * RCSId:       $Id: connection.h,v 1.11 1997/02/13 23:00:06 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -132,5 +132,8 @@ void cxnSetCheckThresholds (Connection cxn,
 void gPrintCxnInfo (FILE *fp, u_int indentAmt) ;
 void printCxnInfo (Connection cxn, FILE *fp, u_int indentAmt) ;
 
+/* config file load callback */
+int cxnConfigLoadCbk (void *data) ;
+
 #endif /* connection_h__ */
 
