@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 23:08:24 1995
  * Project:     INN (innfeed)
  * File:        endpoint.c
- * RCSId:       $Id: endpoint.c,v 1.22 1997/02/14 02:19:47 brister Exp $
+ * RCSId:       $Id: endpoint.c,v 1.24 1997/03/03 12:45:59 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -38,7 +38,7 @@
  */
 
 #if ! defined (lint)
-static char rcsid [] = "$Id: endpoint.c,v 1.22 1997/02/14 02:19:47 brister Exp $" ;
+static char rcsid [] = "$Id: endpoint.c,v 1.24 1997/03/03 12:45:59 brister Exp $" ;
 static void use_rcsid (const char *rId) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rId) ;
 } 
@@ -60,7 +60,7 @@ static void use_rcsid (const char *rId) {   /* Never called */
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#if ! defined (DO_NEED_SYS_SELECT) 
+#if defined (DO_NEED_SYS_SELECT) 
 #include <sys/select.h>
 #endif
 
@@ -78,13 +78,12 @@ static void use_rcsid (const char *rId) {   /* Never called */
 #endif
 
 
-
+#include "configfile.h"
 #include "endpoint.h"
 #include "buffer.h"
 #include "msgs.h"
 
 
-
 #if defined (__bsdi__) && (defined (_ANSI_SOURCE) || defined (_POSIX_SOURCE))
   /* why do I have to do this???? */
 struct timeval
@@ -163,9 +162,9 @@ int debugWrites ;
 
 extern const char *InputFile ;
 
-static int sigFlag ;
 static EndPoint mainEndPoint ;
 static bool mainEpIsReg = false ;
+static u_int stdioFdMax = MAX_STDIO_FD ;
 
 time_t  PrivateTime;
 
@@ -173,6 +172,9 @@ time_t  PrivateTime;
 typedef void (*sigfn) (int) ;
 static sigfn *sigHandlers ;
 
+/* XXX - should be sig_atomic_t if we have it */
+static VOLATILE int *sigFlags ;
+
 
 
   /* private functions */
@@ -187,6 +189,13 @@ static TimerElem newTimerElem (TimeoutId i, time_t w, EndpTCB f, void *d) ;
 static TimeoutId timerElemAdd (time_t when, EndpTCB func, void *data) ;
 static struct timeval *getTimeout (struct timeval *tout) ;
 static void doTimeout (void) ;
+static void handleSignals (void) ;
+
+#if 0
+static int ff_set (fd_set *set, u_int start) ;
+#endif
+static int ff_free (fd_set *set, u_int start) ;
+static void endpointCleanup (void) ;
 
 
   /* Private data */
@@ -195,6 +204,7 @@ static size_t maxEndPoints ;
 static EndPoint *endPoints ;    /* endpoints indexed on fd */
 static EndPoint *priorityList ; /* endpoints indexed on priority */
 
+static int absHighestFd = 0 ;       /* never goes down */
 static int highestFd = -1 ;
 static u_int endPointCount = 0 ;
 static u_int priorityCount = 0 ;
@@ -202,6 +212,7 @@ static u_int priorityCount = 0 ;
 static fd_set rdSet ;
 static fd_set wrSet ;
 static fd_set exSet ;
+static fd_set usedFds ;
 
 static int keepSelecting ;
 
@@ -216,12 +227,93 @@ static int timeoutQueueLength ;
      fields are initialized to appropriate values.  On the first time this
      function is called the global data structs that manages lists of
      endpoints are intialized. */
+static bool inited = false ;
+
 EndPoint newEndPoint (int fd) 
 {
   EndPoint ep ;
 
+  if (!inited)
+    {
+      inited = true ;
+      atexit (endpointCleanup) ;
+    }
+  
   if (fd < 0)
     return NULL ;
+
+  /* try to dup the fd to a larger number to leave lower values free for
+     broken stdio implementations. */
+  if (stdioFdMax > 0 && ((u_int) fd) <= stdioFdMax)
+    {
+      int newfd = ff_free (&usedFds,stdioFdMax + 1) ;
+      if (newfd >= 0)
+        {
+          dprintf (1,"Dupping fd %d to %d\n",fd,newfd) ;
+          if (dup2 (fd,newfd) == -1)
+            {
+              syslog (LOG_ERR,DUP2_FAILED,fd,newfd) ;
+              newfd = fd ;
+            }
+          else if (close (fd) != 0)
+            syslog (LOG_ERR,CLOSE_FAILED,fd) ;
+        }
+      else
+        {
+          syslog (LOG_ERR,NO_FD_FREE,fd,stdioFdMax) ;
+          newfd = fd ;
+        }
+
+      fd = newfd ;
+    }
+
+  if ((u_int) fd >= maxEndPoints)
+    {
+      u_int i = maxEndPoints ;
+      
+      maxEndPoints = (((fd + 256) / 256) * 256); /* round up to nearest 256 */ 
+      if (endPoints == NULL)
+        {
+          endPoints = ALLOC (EndPoint,maxEndPoints) ;
+          priorityList = ALLOC (EndPoint,maxEndPoints) ;
+        }
+      else
+        {
+          endPoints = REALLOC (endPoints,EndPoint,maxEndPoints) ;
+          priorityList = REALLOC (priorityList,EndPoint,maxEndPoints) ;
+        }
+
+      for ( ; i < maxEndPoints ; i++)
+        endPoints [i] = priorityList [i] = NULL ;
+    }
+  
+  ASSERT (endPoints [fd] == NULL) ;
+
+  if (fd > absHighestFd)
+    {
+      static bool sizelogged = false ;
+          
+#if defined (FD_SETSIZE)
+      if (fd >= FD_SETSIZE)
+        {
+          sizelogged = true ;
+          syslog (LOG_ERR,FD_TOO_BIG,fd,FD_SETSIZE,"FD_SETSIZE") ;
+          return NULL ;
+        }
+#else
+      if (fd > (sizeof (fd_set) * CHAR_BIT))
+        {
+          sizelogged = true ;
+          syslog (LOG_ERR,FD_TOO_BIG,fd,(sizeof (fd_set) * CHAR_BIT),
+                  "(sizeof (fd_set) * CHAR_BIT)");
+          return NULL ;
+        }
+#endif
+
+      absHighestFd = fd ;
+    }
+      
+  FD_SET (fd,&usedFds) ;
   
   ep = CALLOC (struct endpoint_s, 1) ;
   ASSERT (ep != NULL) ;
@@ -251,28 +343,6 @@ EndPoint newEndPoint (int fd)
   ep->timesSelected = 1 ;
   ep->timesReady = 0 ;
 
-  if (endPoints == NULL)        /* happens only the first time through */
-    {
-      u_int i ;
-      
-      maxEndPoints = maxFds () ;
-      endPoints = CALLOC (EndPoint,maxEndPoints) ;
-      ASSERT (endPoints != NULL) ;
-      addPointerFreedOnExit ((char *)endPoints) ;
-
-      priorityList = CALLOC (EndPoint,maxEndPoints) ;
-      ASSERT (priorityList != NULL) ;
-      addPointerFreedOnExit ((char *)priorityList) ;
-
-
-      for (i = 0 ; i < maxEndPoints ; i++)
-        endPoints [i] = priorityList [i] = NULL ;
-    }
-
-  ASSERT ((u_int) fd < maxEndPoints) ;
-  ASSERT (endPoints [fd] == NULL) ;
-  ASSERT (priorityCount < maxEndPoints) ;
-  
   endPoints [fd] = ep ;
   priorityList [priorityCount++] = ep ;
   endPointCount++ ;
@@ -307,6 +377,8 @@ void delEndPoint (EndPoint ep)
 
   close (ep->myFd) ;
 
+  FD_CLR (ep->myFd,&usedFds) ;
+  
   /* remove from selectable bits */
   FD_CLR (ep->myFd,&rdSet) ;
   FD_CLR (ep->myFd,&wrSet) ;
@@ -441,18 +513,37 @@ int prepareWrite (EndPoint endp,
 }
 
 
+/* Cancel the pending read. */
+void cancelRead (EndPoint endp)
+{
+  FD_CLR (endp->myFd,&rdSet) ;
+  if (!FD_ISSET (endp->myFd, &wrSet))
+    FD_CLR (endp->myFd,&exSet) ;
+
+  freeBufferArray (endp->inBuffer) ;
+  
+  endp->inBuffer = NULL ;
+  endp->inBufferIdx = 0 ;
+  endp->inIndex = 0 ;
+  endp->inMinLen = 0 ;
+  endp->inAmtRead = 0 ;
+  endp->inCbk = NULL ;
+  endp->inClientData = NULL ;
+}
+
+
 /* cancel all pending writes. The first len bytes of the queued write are
   copied to buffer. The number of bytes copied (if it is less than *len) is
   copied to len. If no write was outstanding then len will have 0 stored in
   it. */
 void cancelWrite (EndPoint endp, char *buffer, size_t *len)
 {
-  char *p ;
-  size_t l ;
+  (void) buffer ;
+  (void) len ;
   
   FD_CLR (endp->myFd, &wrSet) ;
   if (!FD_ISSET (endp->myFd, &rdSet))
-    FD_CLR (endp->myFd, &wrSet) ;
+    FD_CLR (endp->myFd, &exSet) ;
 
 #if 0
 #error XXX need to copy data to buffer and *len
@@ -460,6 +551,7 @@ void cancelWrite (EndPoint endp, char *buffer, size_t *len)
   
   freeBufferArray (endp->outBuffer) ;
   
+  endp->outBuffer = NULL ;
   endp->outBufferIdx = 0 ;
   endp->outIndex = 0 ;
   endp->outCbk = NULL ;
@@ -588,13 +680,7 @@ void Run (void)
         die ("No fd's ready and no timeouts") ;
       else if (sval < 0 && errno == EINTR)
         {
-          if (sigFlag > 0 && sigFlag < NSIG &&
-              sigHandlers[sigFlag] != NULL &&
-              sigHandlers[sigFlag] != SIG_IGN &&
-              sigHandlers[sigFlag] != SIG_DFL)
-            (sigHandlers[sigFlag])(sigFlag) ;
-
-          sigFlag = 0 ;
+	  handleSignals () ;
         }
       else if (sval < 0) 
         {
@@ -607,17 +693,8 @@ void Run (void)
           int readyCount = sval ;
           int timesThrough = 0 ;
           
-          if (sigFlag > 0 && sigFlag < NSIG &&
-              sigHandlers[sigFlag] != NULL &&
-              sigHandlers[sigFlag] != SIG_IGN &&
-              sigHandlers[sigFlag] != SIG_DFL)
-            (sigHandlers[sigFlag])(sigFlag) ;  
-          else if (sigFlag > 0 && sigFlag < NSIG)
-            syslog (LOG_WARNING,"ME Unhandled signal: %d\n",sigFlag) ;
-          else if (sigFlag > NSIG)
-            syslog (LOG_WARNING,"ME signal number too big: %d vs %d\n",
-                    sigFlag,NSIG) ;
-              
+          handleSignals() ;
+          
           for (idx = 0 ; idx < priorityCount ; idx++)
             {
               EndPoint ep = priorityList [idx] ;
@@ -970,19 +1047,13 @@ static IoStatus doRead (EndPoint endp)
         }
       else if (i < 0 && errno == EINTR)
         {
-          if (sigFlag > 0 && sigFlag < NSIG &&
-              sigHandlers[sigFlag] != NULL &&
-              sigHandlers[sigFlag] != SIG_IGN &&
-              sigHandlers[sigFlag] != SIG_DFL)
-            (sigHandlers[sigFlag])(sigFlag) ;
-
-          sigFlag = 0 ;
+	  handleSignals () ;
         }
       else if (i == 0)
         rval = IoEOF ;
       else                   /* i < 0 && errno == EAGAIN */
         rval = IoIncomplete ;
-
+      
       FREE (vp) ;
     }
   else
@@ -1075,13 +1146,7 @@ static IoStatus doWrite (EndPoint endp)
         }
       else if (i < 0 && errno == EINTR)
         {
-          if (sigFlag > 0 && sigFlag < NSIG &&
-              sigHandlers[sigFlag] != NULL &&
-              sigHandlers[sigFlag] != SIG_IGN &&
-              sigHandlers[sigFlag] != SIG_DFL)
-            (sigHandlers[sigFlag])(sigFlag) ;
-          
-          sigFlag = 0 ;
+	  handleSignals () ;
         }
       else if (i < 0 && errno == EAGAIN)
         {
@@ -1138,8 +1203,10 @@ static void endPointPrint (EndPoint ep, FILE *fp)
 
 static void signalHandler (int s)
 {
-  sigFlag = s ;
+  sigFlags[s] = 1 ;
+#if !defined(USE_SIGACTION) && !defined(USE_SIGVEC) && !defined(USE_SIGSET)
   signal (s, signalHandler) ;
+#endif
 }
 
 
@@ -1548,18 +1615,22 @@ int main (int argc, char **argv)
 
   return 0;
 }
-#endif
-
+#endif /* WANT_MAIN */
 
+/* Probably doesn't do the right thing for SIGCHLD */
 void setSigHandler (int signum, void (*ptr)(int))
 {
-  int i ;
-  
+  u_int i ;
+
   if (sigHandlers == NULL)
     {
       sigHandlers = ALLOC (sigfn,NSIG) ;
+      sigFlags = ALLOC (int,NSIG) ;
       for (i = 0 ; i < NSIG ; i++)
-        sigHandlers [i] = NULL ;
+        {
+          sigHandlers [i] = NULL ;
+          sigFlags [i] = 0 ;
+        }
     }
 
   if (signum >= NSIG)
@@ -1569,7 +1640,167 @@ void setSigHandler (int signum, void (*ptr)(int))
       return ;
     }
 
-  (void) signal (signum,signalHandler) ;
+#if defined(USE_SIGACTION)
+  {
+    struct sigaction sa ;
+
+    sa.sa_handler = signalHandler ;
+    sa.sa_flags = 0 ;
+    sigemptyset (&sa.sa_mask) ;
+
+    if (sigaction ( signum, &sa, (struct sigaction *)NULL) != 0)
+      die ("sigaction failed: %s", strerror(errno)) ;
+  }
+#elif defined(USE_SIGVEC)
+  {
+    struct sigvec vec;
+
+    vec.sv_handler = signalHandler ;
+    vec.sv_mask = 0 ;
+    vec.sv_flags = 0 ;
+
+    if (sigvec (signum, &vec, (struct sigvec *)NULL) != 0)
+      die ("sigvec failed: %s", strerror(errno)) ;
+  }
+#elif defined(USE_SIGSET)
+  if (sigset (signum, signalHandler) != 0)
+    die ("sigset failed: %s", strerror(errno)) ;
+#else
+  if (signal (signum, signalHandler) != 0)
+    die ("signal failed: %s", strerror(errno)) ;
+#endif
   sigHandlers[signum] = ptr ;
 }
 
+static void handleSignals (void)
+{
+  int i ;
+#if defined(USE_SIGACTION)
+  sigset_t set, oset ;
+#elif defined(USE_SIGVEC)
+  int mask ;
+#endif
+
+  for (i = 1; i < NSIG; i++)
+    {
+      if (sigFlags[i])
+        {
+#if defined(USE_SIGACTION)
+          sigset_t set, oset ;
+      
+          if (sigemptyset (&set) != 0 || sigaddset (&set, i) != 0)
+            die ("sigemptyset or sigaddset failed") ;
+          if (sigprocmask (SIG_BLOCK, &set, &oset) != 0)
+            die ("sigprocmask failed: %s", strerror(errno)) ;
+#elif defined(USE_SIGVEC)
+# ifndef sigmask
+#  define sigmask(s)    (1 << ((s) - 1))
+# endif
+          int mask ;
+          
+          mask = sigblock (sigmask(i)) ;
+#elif defined(USE_SIGSET)
+          if (sighold (i) != 0)
+            die ("sighold failed: %s", strerror(errno)) ;
+#else
+      /* hope for the best */
+#endif
+
+          sigFlags[i] = 0;
+
+          if (sigHandlers[i] != NULL &&
+              sigHandlers[i] != SIG_IGN &&
+              sigHandlers[i] != SIG_DFL)
+            (sigHandlers[i])(i) ;
+            
+#if defined(USE_SIGACTION)
+          if (sigprocmask (SIG_SETMASK, &oset, (sigset_t *)NULL) != 0)
+            die ("sigprocmask failed: %s", strerror(errno)) ;
+#elif defined(USE_SIGVEC)
+          (void) sigsetmask (mask) ;
+#elif defined(USE_SIGSET)
+          if (sigrelse (i) != 0)
+            die ("sigrelse failed: %s", strerror(errno)) ;
+#else
+          /* hope for the best */
+#endif
+        }
+    }
+}
+
+
+int endpointConfigLoadCbk (void *data)
+{
+  FILE *fp = (FILE *) data ;
+  long ival ;
+  int rval = 1 ;
+
+  if (getInteger (topScope,"stdio-fdmax",&ival,NO_INHERIT))
+    {
+      stdioFdMax = ival ;
+
+#if ! defined (FD_SETSIZE)
+
+      if (stdioFdMax > 0)
+        {
+          logOrPrint (LOG_ERR,fp,NO_STDIO_FDMAX) ;
+          stdioFdMax = 0 ;
+          rval = 0 ;
+        }
+
+#else
+
+      if (stdioFdMax > FD_SETSIZE)
+        {
+          logOrPrint (LOG_ERR,fp,INT_TO_HIGH,"stdio-fdmax",ival,"global scope",
+                      (long) FD_SETSIZE, (long) FD_SETSIZE) ;
+          stdioFdMax = FD_SETSIZE ;
+          rval = 0 ;
+        }
+      
+#endif
+      
+    }
+  else
+    stdioFdMax = 0 ;
+
+  return rval ;
+}
+
+
+
+#if 0
+/* definitely not the fastest, but the most portable way to find the first
+  set bit in a mask  */
+static int ff_set (fd_set *set,u_int start)
+{
+  u_int i ;
+
+  for (i = start ; i < FD_SETSIZE ; i++)
+    if (FD_ISSET (i,set))
+      return (int) i ;
+
+  return -1 ;
+}
+#endif
+
+
+static int ff_free (fd_set *set, u_int start)
+{
+  u_int i ;
+
+  for (i = start ; i <= FD_SETSIZE ; i++)
+    if (!FD_ISSET (i,set))
+      return i ;
+
+
+  return -1 ;
+}
+
+
+static void endpointCleanup (void)
+{
+  FREE (endPoints) ;
+  FREE (priorityList) ;
+  FREE (sigHandlers) ;
+}
