@@ -4,7 +4,7 @@
  * Start Date:  Wed Nov 29 22:04:49 1995
  * Project:     INN (innfeed)
  * File:        endpoint.h
- * RCSId:       $Id: endpoint.h,v 1.7 1997/02/14 02:20:18 brister Exp $
+ * RCSId:       $Id: endpoint.h,v 1.8 1997/03/03 12:48:25 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -133,6 +133,9 @@ int prepareWrite (EndPoint endp,
                   EndpRWCB func,
                   void *clientData) ;
 
+/* cancel any outstanding reads. */
+void cancelRead (EndPoint endp) ;
+
 /* cancel any outstanding writes. */
 void cancelWrite (EndPoint endp, char *buffer, size_t *len) ;
 
@@ -180,6 +183,8 @@ void setMainEndPoint (EndPoint endp) ;
 
 void freeTimeoutQueue (void) ;
 
+int endpointConfigLoadCbk (void *data) ;
+
 
 /*
  * kre's cool idea for reducing the number of times time() is called.
