@@ -4,7 +4,7 @@
  * Start Date:  Thu Dec 28 17:29:05 1995
  * Project:     INN (innfeed)
  * File:        host.c
- * RCSId:       $Id: host.c,v 1.26 1997/02/14 02:23:02 brister Exp $
+ * RCSId:       $Id: host.c,v 1.28 1997/03/03 12:53:15 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: host.c,v 1.26 1997/02/14 02:23:02 brister Exp $" ;
+static const char *rcsid = "$Id: host.c,v 1.28 1997/03/03 12:53:15 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -77,6 +77,7 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #define VALUE_WRONG_TYPE 4
 
 extern char *configFile ;
+extern mainLogStatus (FILE *fp) ;
 
 /* the host keeps a couple lists of these */
 typedef struct proc_q_elem 
@@ -213,6 +214,7 @@ static double defaultBacklogFactor ;
 static bool factorSet ;
 
 static HostHolder blockedHosts ; /* lists of hosts we can't lock */
+static time_t lastStatusLog ;
 
   /*
    * Host object private methods.
@@ -246,13 +248,8 @@ static bool getHostInfo (char **name,
                          double *highFilter,
                          u_short *portNumber) ;
 static Host findHostByName (char *name) ;
+static void hostCleanup (void) ;
 
-#if 0
-static bool getOptUInt (u_int *field, char *line, u_int fNum, u_int defVal) ;
-static bool getOptBool (bool *field, char *line, u_int fNum, bool defVal) ;
-static bool getOptDouble (double *field, char *line, u_int fieldNum,
-                          double defVal) ;
-#endif
 
 /* article queue management functions */
 static Article remHead (ProcQElem *head, ProcQElem *tail) ;
@@ -294,10 +291,6 @@ static u_int defClosePeriod ;
 
 static bool genHtml = false ;
 
-#if 0
-static bool inited = false ;
-#endif
-
 /*******************************************************************/
 /*                  PUBLIC FUNCTIONS                               */
 /*******************************************************************/
@@ -319,7 +312,8 @@ int hostConfigLoadCbk (void *data)
       if (iv < 1)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,DNSRET_LOW,iv,(long)DNS_RETRY_PERIOD) ;
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ONE,"dns-retry",
+                      iv,"global scope",(long)DNS_RETRY_PERIOD) ;
           iv = DNS_RETRY_PERIOD ;
         }
     }
@@ -333,7 +327,8 @@ int hostConfigLoadCbk (void *data)
       if (iv < 1)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,DNSEXP_LOW,iv,(long)DNS_EXPIRE_PERIOD) ;
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ONE,"dns-expire",iv,
+                      "global scope",(long)DNS_EXPIRE_PERIOD) ;
           iv = DNS_EXPIRE_PERIOD ;
         }
     }
@@ -347,7 +342,8 @@ int hostConfigLoadCbk (void *data)
       if (iv < 1)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,CLOSEPER_LOW,iv,(long)CLOSE_PERIOD) ;
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ONE,"close-period",iv,
+                      "global scope",(long)CLOSE_PERIOD) ;
           iv = CLOSE_PERIOD ;
         }
     }
@@ -363,7 +359,10 @@ int hostConfigLoadCbk (void *data)
   
   
   if (getString (topScope,"status-file",&p,NO_INHERIT))
-    hostSetStatusFile (p) ;
+    {
+      hostSetStatusFile (p) ;
+      FREE (p) ;
+    }
   else
     hostSetStatusFile (INNFEED_STATUS) ;
   
@@ -379,7 +378,8 @@ int hostConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,NEGHOSTHIGH,iv,(long) HOST_HIGHWATER) ;
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"host-queue-highwater",
+                      iv,"global scope",(long) HOST_HIGHWATER) ;
           iv = HOST_HIGHWATER ;
         }
     }
@@ -387,12 +387,13 @@ int hostConfigLoadCbk (void *data)
     iv = HOST_HIGHWATER ;
   hostHighwater = (u_int) iv ;
 
-  if (getInteger (topScope,"stats-period", &iv,NO_INHERIT))
+  if (getInteger (topScope,"stats-period",&iv,NO_INHERIT))
     {
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,NEGSTATPERIOD,iv,(long)STATS_PERIOD) ;
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"stats-period",
+                      iv,"global scope",(long)STATS_PERIOD) ;
           iv = STATS_PERIOD ;
         }
     }
@@ -406,8 +407,8 @@ int hostConfigLoadCbk (void *data)
       if (iv < 0)
         {
           rval = 0 ;
-          logOrPrint (LOG_ERR,fp,NEGRESETPERIOD,iv,
-                      (long)STATS_RESET_PERIOD) ;
+          logOrPrint (LOG_ERR,fp,LESS_THAN_ZERO,"stats-reset",iv,
+                      "global scope",(long)STATS_RESET_PERIOD) ;
           iv = STATS_RESET_PERIOD ;
         }
     }
@@ -537,7 +538,7 @@ int hostConfigLoadCbk (void *data)
 
 void configHosts (bool talkSelf)
 {
-  Host nHost, h ;
+  Host nHost, h, q ;
   char *name = NULL, *ipName = NULL ;
   u_int artTout, respTout, initCxns, maxCxns, maxChecks ;
   double lowFilter, highFilter ;
@@ -559,8 +560,6 @@ void configHosts (bool talkSelf)
   closeDroppedArticleFile () ;
   openDroppedArticleFile () ;
 
-  syslog(LOG_NOTICE, "loading %s", configFile) ;
-
   while (getHostInfo (&name, &ipName, &artTout, &respTout, &initCxns,
                       &maxCxns, &maxChecks, &streaming, &lowFilter,
                       &highFilter,&portNum))
@@ -602,7 +601,8 @@ void configHosts (bool talkSelf)
 			 hostCxnDead doesn't kill the process, the
 			 ASSERT against h->maxConnections surely will.
 		       */
-		      cxnNuke (h->connections[i-1]) ;
+                      if (h->connections[i - 1] != NULL)
+                        cxnNuke (h->connections[i-1]) ;
 		    }
 		  h->maxConnections = maxCxns ;
 		}
@@ -644,7 +644,8 @@ void configHosts (bool talkSelf)
 	    }
 
 	  for ( i = 0 ; i < maxCxns ; i++ )
-	    cxnSetCheckThresholds (h->connections[i], lowFilter, highFilter) ;
+            if (h->connections[i] != NULL)
+              cxnSetCheckThresholds (h->connections[i],lowFilter,highFilter) ;
 
 	  /* XXX how to handle initCxns change? */
         }
@@ -653,7 +654,7 @@ void configHosts (bool talkSelf)
 	  nHost = newHost (mainListener,name,ipName,artTout,respTout,initCxns,
 			   maxCxns,maxChecks,portNum,CLOSE_PERIOD,
 			   streaming,lowFilter,highFilter) ;
-	  
+
 	  if (nHost == NULL)
 	    {
               /* this is where we'll die if the locks are still held. */
@@ -675,7 +676,7 @@ void configHosts (bool talkSelf)
               hh->next = blockedHosts ;
               blockedHosts = hh ;
                
-	      syslog (LOG_ERR,NO_HOST,name) ;
+	      syslog (LOG_ERR,NO_HOST,hh->name) ;
 	    }
 	  else 
 	    {
@@ -686,7 +687,10 @@ void configHosts (bool talkSelf)
 		       name, ipName, artTout, respTout, initCxns, maxCxns,
 		       maxChecks, portNum, streaming ? "true" : "false",
 		       lowFilter, highFilter) ;
-	      
+
+              FREE (name) ;
+              FREE (ipName) ;
+              
 	      if ( !listenerAddPeer (mainListener,nHost) )
 		die ("failed to add a new peer\n") ;
 	    }
@@ -696,12 +700,15 @@ void configHosts (bool talkSelf)
     }
   
 
-  for (h = gHostList; h != NULL; h = h->next)
-    if (h->removeOnReload)
-      hostClose (h) ;
-    else
-      /* prime it for the next config file read */
-      h->removeOnReload = true ;
+  for (h = gHostList; h != NULL; h = q) 
+    {
+      q = h->next ;
+      if (h->removeOnReload)
+        hostClose (h) ;         /* h may be deleted in here. */
+      else
+        /* prime it for the next config file read */
+        h->removeOnReload = true ;
+    }
 
   hostLogStatus () ;
 }
@@ -710,6 +717,7 @@ void configHosts (bool talkSelf)
  * Create a new Host object. Called by the InnListener.
  */
 
+static bool inited = false ;
 Host newHost (InnListener listener,
               const char *name, 
               const char *ipName,
@@ -727,9 +735,15 @@ Host newHost (InnListener listener,
   u_int i ;
   Host nh ; 
 
-  ASSERT (maxCxns > 0 && maxCxns < MAX_CONNECTION_COUNT) ;
+  ASSERT (maxCxns > 0) ;
   ASSERT (maxCheck > 0) ;
 
+  if (!inited)
+    {
+      inited = true ;
+      atexit (hostCleanup) ;
+    }
+  
 
   nh =  CALLOC (struct host_s, 1) ;
   ASSERT (nh != NULL) ;
@@ -1056,12 +1070,13 @@ void printHostInfo (Host host, FILE *fp, u_int indentAmt)
   
   
   fprintf (fp,"%s    Connections {\n",indent) ;
-  for (cxn = host->connections ; cxn != NULL && *cxn != NULL ; cxn++)
+  for (i = 0 ; i < host->maxConnections ; i++)
     {
 #if 0
-    printCxnInfo (*cxn,fp,indentAmt + INDENT_INCR) ;
+      if (host->connections[i] != NULL)
+        printCxnInfo (*cxn,fp,indentAmt + INDENT_INCR) ;
 #else
-    fprintf (fp,"%s        %p\n",indent,*cxn) ;
+      fprintf (fp,"%s        %p\n",indent,host->connections[i]) ;
 #endif
     }
   fprintf (fp,"%s    }\n",indent) ;
@@ -1096,6 +1111,8 @@ void printHostInfo (Host host, FILE *fp, u_int indentAmt)
 void hostClose (Host host)
 {
   u_int i ;
+  u_int mxCxns ;
+  u_int cxnCount ;
 
   dprintf (1,"Closing host %s\n",host->peerName) ;
   
@@ -1109,8 +1126,16 @@ void hostClose (Host host)
   
   host->connectTime = 0 ;
 
-  for (i = 0 ; i < host->maxConnections ; i++)
-    cxnTerminate (host->connections [i]) ;
+  /* when we call cxnTerminate() on the last Connection, the Host objects
+     will end up getting deleted out from under us (via hostCxnGone()). If
+     we are running with a malloc that scribbles over memory after freeing
+     it, then we'd fail in the second for loop test. Trying to access
+     host->maxConnections. */
+  for (i = 0, cxnCount = 0 ; i < host->maxConnections ; i++) 
+    cxnCount += (host->connections [i] != NULL ? 1 : 0) ;
+  for (i = 0 ; i < cxnCount ; i++)
+    if (host->connections[i] != NULL)
+      cxnTerminate (host->connections [i]) ;
 }
 
 
@@ -1762,15 +1787,6 @@ bool hostLogConnectionStatsP (void)
 }
 
 
-#if 0
-/*
- * Set the number of seconds between each logging done by the Hosts.
- */
-void hostSetStatsPeriod (u_int period) 
-{
-  statsPeriod = period ;
-}
-#endif
 
 /*
  * Called by one of the Host's Connection's when it (the Connection)
@@ -1863,12 +1879,12 @@ static void hostDetails (scope *s,
       
   if (!getInteger (s,ARTICLE_TIMEOUT,&iv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,ARTICLE_TIMEOUT,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_INT,ARTICLE_TIMEOUT,name,(long) ARTTOUT) ;
       iv = ARTTOUT ;
     }
-  else if (iv <= 0)
+  else if (iv < 1)
     {
-      syslog (LOG_ERR,NON_POSITIVE_ERR,ARTICLE_TIMEOUT,iv,name) ;
+      syslog (LOG_ERR,LESS_THAN_ONE,ARTICLE_TIMEOUT,iv,name,(long)ARTTOUT) ;
       iv = ARTTOUT ;
     }
   *articleTimeout = (u_int) iv ;
@@ -1877,12 +1893,12 @@ static void hostDetails (scope *s,
   
   if (!getInteger (s,RESP_TIMEOUT,&iv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,RESP_TIMEOUT,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_INT,RESP_TIMEOUT,name,(long)RESPTOUT) ;
       iv = RESPTOUT ;
     }
-  else if (iv <= 0)
+  else if (iv < 1)
     {
-      syslog (LOG_ERR,NON_POSITIVE_ERR,RESP_TIMEOUT,iv,name) ;
+      syslog (LOG_ERR,LESS_THAN_ONE,RESP_TIMEOUT,iv,name,(long)RESPTOUT) ;
       iv = RESPTOUT ;
     }
   *responseTimeout = (u_int) iv ;
@@ -1891,12 +1907,14 @@ static void hostDetails (scope *s,
 
   if (!getInteger (s,INITIAL_CONNECTIONS,&iv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,INITIAL_CONNECTIONS,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_INT,INITIAL_CONNECTIONS,name,
+              (long)INIT_CXNS) ;
       iv = INIT_CXNS ;
     }
   else if (iv < 0)
     {
-      syslog (LOG_ERR,LESS_THAN_ZERO,INITIAL_CONNECTIONS,iv,name);
+      syslog (LOG_ERR,LESS_THAN_ZERO,INITIAL_CONNECTIONS,iv,
+              name,(long)INIT_CXNS);
       iv = INIT_CXNS ;
     }
   *initialConnections = (u_int) iv ;
@@ -1905,12 +1923,17 @@ static void hostDetails (scope *s,
 
   if (!getInteger (s,MAX_CONNECTIONS,&iv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,MAX_CONNECTIONS,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_INT,MAX_CONNECTIONS,name,(long)MAX_CXNS) ;
       iv = MAX_CXNS ;
     }
   else if (iv < 1)
     {
-      syslog (LOG_ERR,LESS_THAN_ZERO,MAX_CONNECTIONS,iv,name) ;
+      syslog (LOG_ERR,LESS_THAN_ONE,MAX_CONNECTIONS,iv,name,(long)1) ;
+      iv = 1 ;
+    }
+  else if (iv > MAX_CONNECTION_COUNT)
+    {
+      syslog (LOG_ERR,INT_TOO_LARGE,MAX_CONNECTIONS,iv,name,(long)MAX_CXNS) ;
       iv = MAX_CXNS ;
     }
   *maxConnections = (u_int) iv ;
@@ -1919,12 +1942,12 @@ static void hostDetails (scope *s,
 
   if (!getInteger (s,MAX_QUEUE_SIZE,&iv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,MAX_QUEUE_SIZE,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_INT,MAX_QUEUE_SIZE,name,(long)MAX_Q_SIZE) ;
       iv = MAX_Q_SIZE ;
     }
   else if (iv < 1)
     {
-      syslog (LOG_ERR,NON_POSITIVE_ERR,MAX_QUEUE_SIZE,iv,name) ;
+      syslog(LOG_ERR,LESS_THAN_ONE,MAX_QUEUE_SIZE,iv,name,(long)MAX_Q_SIZE);
       iv = MAX_Q_SIZE ;
     }
   *maxChecks = (u_int) iv ;
@@ -1933,7 +1956,7 @@ static void hostDetails (scope *s,
 
   if (!getBool (s,STREAMING,&bv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,STREAMING,name) ;
+      syslog(LOG_ERR,NO_PEER_FIELD_BOOL,STREAMING,name,(STREAM ? "true" : "false"));
       bv = STREAM ;
     }
   *streaming = (bv ? true : false) ;
@@ -1942,7 +1965,7 @@ static void hostDetails (scope *s,
 
   if (!getReal (s,NO_CHECK_LOW,&rv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,NO_CHECK_LOW,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_REAL,NO_CHECK_LOW,name,NOCHECKLOW) ;
       rv = NOCHECKLOW ;
     }
   else if (rv < 0.0 || rv > 100.0)
@@ -1957,7 +1980,7 @@ static void hostDetails (scope *s,
 
   if (!getReal (s,NO_CHECK_HIGH,&rv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,NO_CHECK_HIGH,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_REAL,NO_CHECK_HIGH,name,NOCHECKHIGH) ;
       rv = NOCHECKHIGH ;
     }
   else if (rv < 0.0 || rv > 100.0)
@@ -1979,7 +2002,7 @@ static void hostDetails (scope *s,
 
   if (!getInteger (s,PORT_NUMBER,&iv,INHERIT))
     {
-      syslog (LOG_ERR,NO_PEER_FIELD,PORT_NUMBER,name) ;
+      syslog (LOG_ERR,NO_PEER_FIELD_INT,PORT_NUMBER,name,(long)PORTNUM) ;
       iv = PORTNUM ;
     }
   else if (iv < 1)
@@ -2010,23 +2033,10 @@ static bool getHostInfo (char **name,
   value *v ;
   scope *s ;
 
-  char *iname ;
-  long tout, rtout, icxns, maxcxns, maxcks, portnum, blog ;
-  int strm ;
-  double lfil, hfil ;
   bool isGood = false ;
 
   if (topScope == NULL)
     return false ;
-
-#if 0
-  
-  tout = rtout = icxns = maxcxns = maxcks = portnum = blog = 0 ;
-  iname = NULL ;
-  strm = false ;
-  lfil = hfil = 0.0 ;
-
-#endif
   
   while ((v = getNextPeer (&idx)) != NULL) 
     {
@@ -2036,9 +2046,6 @@ static bool getHostInfo (char **name,
       s = v->v.scope_val ;
       *name = strdup (v->name) ;
 
-
-#if 1
-
       hostDetails(s,*name,ipName,articleTimeout,responseTimeout,
                   initialConnections,maxConnections,maxChecks,streaming,
                   lowFilter,highFilter,portNumber) ;
@@ -2046,191 +2053,6 @@ static bool getHostInfo (char **name,
       isGood = true ;
       
       break ;
-      
-#else
-
-      isGood = true ;
-
-      /*
-      **
-      */
-      if (!getString (s,IP_NAME,&iname,NO_INHERIT))
-        iname = *name ;
-
-      
-      /*
-      **
-      */
-      if (!getInteger (s,ARTICLE_TIMEOUT,&tout,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,ARTICLE_TIMEOUT,v->name) ;
-          isGood = false ;
-        }
-      else if (tout <= 0)
-        {
-          syslog (LOG_ERR,NON_POSITIVE_ERR,ARTICLE_TIMEOUT,tout,v->name) ;
-          isGood = false ;
-        }
-      
-
-      /*
-      **
-      */
-      if (!getInteger (s,RESP_TIMEOUT,&rtout,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,RESP_TIMEOUT,v->name) ;
-          isGood = false ;
-        }
-      else if (rtout <= 0)
-        {
-          syslog (LOG_ERR,NON_POSITIVE_ERR,RESP_TIMEOUT,rtout,v->name) ;
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getInteger (s,INITIAL_CONNECTIONS,&icxns,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,INITIAL_CONNECTIONS,v->name) ;
-          isGood = false ;
-        }
-      else if (icxns < 0)
-        {
-          syslog (LOG_ERR,LESS_THAN_ZERO,INITIAL_CONNECTIONS,icxns,v->name);
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getInteger (s,MAX_CONNECTIONS,&maxcxns,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,MAX_CONNECTIONS,v->name) ;
-          isGood = false ;
-        }
-      else if (maxcxns < 1)
-        {
-          syslog (LOG_ERR,LESS_THAN_ZERO,MAX_CONNECTIONS,maxcxns,v->name) ;
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getInteger (s,MAX_QUEUE_SIZE,&maxcks,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,MAX_QUEUE_SIZE,v->name) ;
-          isGood = false ;
-        }
-      else if (maxcks < 1)
-        {
-          syslog (LOG_ERR,NON_POSITIVE_ERR,MAX_QUEUE_SIZE,maxcks,v->name) ;
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getBool (s,STREAMING,&strm,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,STREAMING,v->name) ;
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getReal (s,NO_CHECK_LOW,&lfil,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,NO_CHECK_LOW,v->name) ;
-          isGood = false ;
-        }
-      else if (lfil < 0.0 || lfil > 100.0)
-        {
-          syslog (LOG_ERR,
-                  "ME %s value (%.2f) in peer %s must be in range [0.0, 100.0]",
-                  NO_CHECK_LOW,lfil,v->name) ;
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getReal (s,NO_CHECK_HIGH,&hfil,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,NO_CHECK_HIGH,v->name) ;
-          isGood = false ;
-        }
-      else if (hfil < 0.0 || hfil > 100.0)
-        {
-          syslog (LOG_ERR,
-                  "ME %s value (%.2f) in peer %s must be in range [0.0, 100.0]",
-                  NO_CHECK_HIGH,hfil,v->name) ;
-          isGood = false ;
-        }
-      else if (isGood && hfil < lfil)
-        {
-          syslog (LOG_ERR,
-                  "ME %s's value (%.2f) in peer %s cannot be smaller than %s's value (%.2f)",
-                  NO_CHECK_HIGH, hfil, v->name, NO_CHECK_LOW, lfil);
-        }
-      
-
-      /*
-      **
-      */
-      if (!getInteger (s,PORT_NUMBER,&portnum,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,PORT_NUMBER,v->name) ;
-          isGood = false ;
-        }
-      else if (portnum < 1)
-        {
-          syslog (LOG_ERR,"ME %s value (%ld) in peer %s cannot be less than 1",
-                  PORT_NUMBER,portnum,v->name) ;
-          isGood = false ;
-        }
-
-      
-      /*
-      **
-      */
-      if (!getInteger (s,BACKLOG_LIMIT,&blog,INHERIT))
-        {
-          syslog (LOG_ERR,NO_PEER_FIELD,BACKLOG_LIMIT,v->name) ;
-          isGood = false ;
-        }
-      else if (blog < 0)
-        {
-          syslog (LOG_ERR,"ME %s value (%ld) in peer %s cannot be less than 0",
-                  BACKLOG_LIMIT,blog,v->name) ;
-          isGood = false ;
-        }
-
-      if (isGood)
-        {
-          *ipName = iname ;
-          *articleTimeout = (u_int) tout ;
-          *responseTimeout = (u_int) rtout ;
-          *initialConnections = (u_int) icxns ;
-          *maxConnections = (u_int) maxcxns ;
-          *maxChecks = (u_int) maxcks ;
-          *portNumber = (u_short) portnum ;
-          
-          *streaming = (strm ? true : false) ;
-          *lowFilter = lfil / 10.0 ; /* config range is [0.0,100.0] */
-          *highFilter = hfil / 10.0 ; /* internally it's [0.0,10.0] */
-
-          break ;
-        }
-#endif
     }
 
   if (v == NULL)
@@ -2251,8 +2073,6 @@ void getHostDefaults (u_int *articleTout,
                       double *highFilter,
                       u_short *portNum)
 {
-  long lv ;
-  
   ASSERT (defaultMaxConnections > 0) ;
   
   ASSERT (articleTout != NULL) ;
@@ -2508,6 +2328,8 @@ static void hostLogStatus (void)
   if (!anyToLog)
     return ;
 
+  lastStatusLog = theTime() ;
+  
   if ((fp = fopen (statusFile,"w")) == NULL)
     {
       if ( !flogged )
@@ -2533,22 +2355,20 @@ static void hostLogStatus (void)
                peerNum, actConn, slpConn,(maxcon - (actConn + slpConn))) ;
 
       fprintf (fp,"Configuration file: %s\n\n",configFile) ;
+
+      mainLogStatus (fp) ;
+      listenerLogStatus (fp) ;
       
-      fprintf (fp,"Default configuration parameters:\n") ;
-
-      fprintf (fp,"    article timeout: %-5d     initial connections: %d\n",
+      fprintf(fp,"Default peer configuration parameters:\n") ;
+      fprintf(fp,"    article timeout: %-5d     initial connections: %d\n",
 	       defaultArticleTimeout, defaultInitialConnections) ;
-
-      fprintf (fp,"   response timeout: %-5d         max connections: %d\n",
+      fprintf(fp,"   response timeout: %-5d         max connections: %d\n",
 	       defaultResponseTimeout, defaultMaxConnections) ;
-
-      fprintf (fp,"         max checks: %-5d               streaming: %s\n",
+      fprintf(fp,"         max checks: %-5d               streaming: %s\n",
 	       defaultMaxChecks, (defaultStreaming ? "true" : "false")) ;
-
-      fprintf (fp,"       on threshold: %-2.1f%%                port num: %d\n",
+      fprintf(fp,"       on threshold: %-2.1f%%                port num: %d\n",
 	       (defaultHighFilter * 10), defaultPortNumber) ;
-
-      fprintf (fp,"      off threshold: %-2.1f%%\n\n",
+      fprintf(fp,"      off threshold: %-2.1f%%\n\n",
 	       (defaultLowFilter * 10)) ;
 
       tapeLogGlobalStatus (fp) ;
@@ -2653,6 +2473,7 @@ static void hostPrintStatus (Host host, FILE *fp)
 static void hostStatsTimeoutCbk (TimeoutId tid, void *data)
 {
   Host host = (Host) data ;
+  time_t now = theTime () ;
 
   (void) tid ;                  /* keep lint happy */
 
@@ -2660,6 +2481,10 @@ static void hostStatsTimeoutCbk (TimeoutId tid, void *data)
   
   if (!amClosing (host))
     hostLogStats (host, false) ;
+
+  if (now - lastStatusLog >= statsPeriod)
+    hostLogStatus () ;
+  
   
   host->statsId = prepareSleep (hostStatsTimeoutCbk, statsPeriod, host) ;
 }
@@ -2852,7 +2677,7 @@ static int validateInteger (FILE *fp, const char *name,
           logOrPrint (LOG_ERR,fp,NODEFN,name) ;
         }
       else if (required)
-        logOrPrint (LOG_INFO,fp,ADDMISSINT,name,setval) ;
+        logOrPrint (LOG_INFO,fp,ADDMISSINGINT,name,setval) ;
     }
   else if (v != NULL && v->type != intval)
     {
@@ -2862,14 +2687,16 @@ static int validateInteger (FILE *fp, const char *name,
   else if (v != NULL && low != LONG_MIN && v->v.int_val < low)
     {
       rval = VALUE_TOO_LOW ;
-      logOrPrint (LOG_ERR,fp,INTTOLOW,name,v->v.int_val,low) ;
-      v->v.int_val = setval ;
+      logOrPrint (LOG_ERR,fp,INT_TO_LOW,name,v->v.int_val,
+                  "global scope",low,low) ;
+      v->v.int_val = low ;
     }
   else if (v != NULL && high != LONG_MAX && v->v.int_val > high)
     {
       rval = VALUE_TOO_HIGH ;
-      logOrPrint (LOG_ERR,fp,INTTOHIGH,name,v->v.int_val,high) ;
-      v->v.int_val = setval ;
+      logOrPrint(LOG_ERR,fp,INT_TO_HIGH,name,v->v.int_val,
+                 "global scope",high,high);
+      v->v.int_val = high ;
     }
   
   return rval ;
@@ -2949,3 +2776,9 @@ static int validateBool (FILE *fp, const char *name, int required, bool setval)
 }
 
 
+static void hostCleanup (void)
+{
+  if (statusFile != NULL)
+    FREE (statusFile) ;
+}
+
