@@ -4,7 +4,7 @@
  * Start Date:  Mon, 15 Jan 1996 17:31:58 +1100
  * Project:     INN -- innfeed
  * File:        main.c
- * RCSId:       $Id: main.c,v 1.24 1997/02/13 23:21:31 brister Exp $
+ * RCSId:       $Id: main.c,v 1.25 1997/03/03 13:06:35 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: main.c,v 1.24 1997/02/13 23:21:31 brister Exp $" ;
+static const char *rcsid = "$Id: main.c,v 1.25 1997/03/03 13:06:35 brister Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -60,6 +60,9 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include <limits.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/resource.h>
+
 
 #include "misc.h"
 #include "tape.h"
@@ -118,6 +121,7 @@ static void openLogFile (void) ;
 static void writePidFile (void) ;
 static int mainOptionsProcess (void *data) ;
 static int mainConfigLoadCbk (void *data) ;
+static void mainCleanup (void) ;
 
 static char *bopt = NULL ;
 static char *aopt = NULL ;
@@ -144,6 +148,8 @@ int main (int argc, char **argv)
   char dateString [30] ;
   char *copt = NULL ;
   bool checkConfig = false ;
+  struct rlimit rl;
+
 
   strcpy (dateString,ctime(&now)) ;
   dateString [24] = '\0' ;
@@ -335,6 +341,7 @@ int main (int argc, char **argv)
   configAddLoadCallback (mainOptionsProcess,(checkConfig ? stderr : NULL)) ;
   configAddLoadCallback (tapeConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
 
+  configAddLoadCallback (endpointConfigLoadCbk,(checkConfig ? stderr : NULL));
   configAddLoadCallback (hostConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
   configAddLoadCallback (cxnConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
   configAddLoadCallback (mainConfigLoadCbk,(checkConfig ? stderr : NULL)) ;
@@ -456,6 +463,28 @@ int main (int argc, char **argv)
   mainListener = listener ;
 
   sleep (initialSleep) ;
+
+
+  /* now lower maximum open file limit to match what select(2) can handle. */
+  if (getrlimit(RLIMIT_NOFILE,&rl) != 0)
+    syslog (LOG_ERR,GETRLIM_FAILED) ;
+  else
+    {
+#if defined (FD_SETSIZE)
+      u_int fd_max = FD_SETSIZE ;
+#else
+      u_int fd_max = sizeof (fd_set) * CHAR_BIT ;
+#endif
+      
+      if (rl.rlim_max > fd_max)
+        {
+          rl.rlim_max = rl.rlim_cur = fd_max ;
+          if (setrlimit (RLIMIT_NOFILE,&rl) != 0)
+            syslog (LOG_ERR,SETRLIM_FAILED,(long)fd_max);
+        }
+    }
+  
+      
   
   configHosts (talkToSelf) ;
 
@@ -463,7 +492,7 @@ int main (int argc, char **argv)
     openInputFile () ;
   }
 
-  /* handle signal to shutndown */
+  /* handle signal to shutdown */
   setSigHandler (SIGTERM,sigterm) ;
   setSigHandler (SIGQUIT,sigquit) ;
 
@@ -483,6 +512,8 @@ int main (int argc, char **argv)
   setSigHandler (SIGUSR1,sigusr) ;
   setSigHandler (SIGUSR2,sigusr) ;
 
+  atexit (mainCleanup) ;
+  
   Run () ;
 
   exit (0) ;
@@ -665,8 +696,6 @@ static void writePidFile (void)
   FILE *F;
   int pid;
 
-  pidFile = buildFilename (getTapeDirectory(),pidFile) ;
-
   if (pidFile == NULL)
     logAndExit (1,"NULL pidFile\n") ;
 
@@ -700,7 +729,7 @@ static void gprintinfo (void)
 
   if (fp == NULL)
     {
-      syslog (LOG_ERR,NO_SNAPSHOT,SNAPSHOT_FILE) ;
+      syslog (LOG_ERR,FOPEN_FAILURE,SNAPSHOT_FILE) ;
       return ;
     }
 
@@ -753,10 +782,12 @@ static int mainConfigLoadCbk (void *data)
   /***************************************************/
   
   if (getString (topScope,"pid-file",&p,NO_INHERIT))
-    pidFile = buildFilename (getTapeDirectory(),p) ;
+    {
+      pidFile = buildFilename (getTapeDirectory(),p) ;
+      free (p) ;
+    }
   else
-    pidFile = PID_FILE ;
-
+    pidFile = buildFilename (getTapeDirectory(),PID_FILE) ;
   
   if (getInteger (topScope,"debug-level",&ival,NO_INHERIT))
     loggingLevel = (u_int) ival ;
@@ -771,7 +802,11 @@ static int mainConfigLoadCbk (void *data)
 
   
   if (getString (topScope,"log-file",&p,NO_INHERIT))
-    logFile = buildFilename (getTapeDirectory(),p) ;
+    {
+      logFile = buildFilename (getTapeDirectory(),p) ;
+      FREE (p) ;
+    }
+  
 
   return 1 ;
 }
@@ -883,3 +918,35 @@ static int mainOptionsProcess (void *data)
 
   return 1 ;
 }
+
+
+
+static void mainCleanup (void)
+{
+  FREE (configFile) ;
+  FREE (pidFile) ;
+  FREE (logFile) ;
+  FREE (newsspool) ;
+}
+
+
+void mainLogStatus (FILE *fp)
+{
+  fprintf (fp,"Global configuration parameters:\n") ;
+  fprintf (fp,"          Mode: ") ;
+  if (InputFile != NULL)
+    fprintf (fp,"Funnel file") ;
+  else if (talkToSelf)
+    fprintf (fp,"Batch") ;
+  else
+    fprintf (fp,"Channel") ;
+  if (InputFile != NULL)
+    fprintf (fp,"   (%s)",(*InputFile == '\0' ? "stdin" : InputFile)) ;
+  fprintf (fp,"\n") ;
+  fprintf (fp,"    News spool: %s\n",newsspool) ;
+  fprintf (fp,"      Pid file: %s\n",pidFile) ;
+  fprintf (fp,"      Log file: %s\n",(logFile == NULL ? "(none)" : logFile));
+  fprintf (fp,"   Debug level: %2ld                Mmap: %s\n",
+           (long)loggingLevel,boolToString(useMMap)) ;
+  fprintf (fp,"\n") ;
+}
