@@ -4,7 +4,7 @@
  * Start Date:  Thu, 01 Feb 1996 22:17:51 +1100
  * Project:     INN -- innfeed
  * File:        sysconfig.h
- * RCSId:       $Id: sysconfig.h,v 1.11 1997/02/13 22:58:38 brister Exp $
+ * RCSId:       $Id: sysconfig.h,v 1.13 1997/03/03 13:21:15 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -83,6 +83,7 @@
 /* Note: If you are running version 5.4.3 or better of libc, then 16 is the
    number to use. Lower than that, and you should use 1. */
 #define MAX_WRITEV_VEC 1
+#define DO_NEED_SYS_SELECT 1
 #endif
 
 
@@ -108,11 +109,11 @@
 #define MAX_WRITEV_VEC  16
 #define DO_NEED_STREAM 1
 #define HAVE_MMAP
+#define MAX_STDIO_FD 256
 
  /* this may be needed for solaris version < 2.6 */
 /* #define wait3(a,b,c) waitpid(-1,a,b) */
 
-
 #endif /* defined (SOLARIS) */
 
 
@@ -124,6 +125,7 @@
 #define MAX_WRITEV_VEC 16
 #define DO_NEED_STRERROR 1
 #define HAVE_MMAP
+#define MAX_STDIO_FD 128
 #endif
 
 
@@ -179,6 +181,23 @@
   /* Defaults below here. If you need to change something it should really
      be done in the architecture specific section just above. */
 
+/* If you compiler doesn't support `volatile' then add a line like
+              #define VOLATILE
+   above */
+#if ! defined (VOLATILE)
+#define VOLATILE volatile
+#endif
+
+
+/* Some broken system (all SunOS versions) have a lower limit for the
+   maximum number of stdio files that can be open, than the limit of open
+   file the OS will let you have. If this value is > 0 (and ``stdio-fdmax''
+   is *not* used in the config file), then all non-stdio file descriptors
+   will be kept above this value (by dup'ing them). */
+#if ! defined (MAX_STDIO_FD)
+#define MAX_STDIO_FD 0
+#endif
+
 /* define DONT_NEED_SYS_SELECT or DO_NEED_SYS_SELECT depending on whether
    endpoint.c fails to compile properly and you have
    /usr/include/sys/select.h */
