@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 09:55:07 1995
  * Project:     INN (innfeed)
  * File:        tape.h
- * RCSId:       $Id: tape.h,v 1.9 1997/02/14 13:08:24 brister Exp $
+ * RCSId:       $Id: tape.h,v 1.10 1997/03/03 13:25:00 brister Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -70,22 +70,6 @@ void gFlushTapes (void) ;
 void tapeFlush (Tape tape) ;
 
 
-#if 0
-/* Opnes the output side of the tape for writing.  Returns false if the
-   underlying fopen() failed, Any articles the Tape has in memory will be
-   written to it and any new articles it gets will go straight to the
-   disk. */
-bool tapeOpen (Tape tape) ;
-
-/* close the Tape's output side. */
-void tapeClose (Tape tape) ;
-
-/* return true if article info on disk or in memory */
-bool tapeHasArticles (Tape tape) ;
-#endif
-
-
-
 /**************************************************/
 /*               CLASS LEVEL FUNCTIONS            */
 /**************************************************/
@@ -93,14 +77,6 @@ bool tapeHasArticles (Tape tape) ;
 /* get all the active input tapes to checkpoint their current positions */
 void checkPointTapes (void) ;
 
-#if 0
-/* set the number of seconds between checkpoints of the tape files. */
-void tapeSetCheckpointPeriod (u_int ckPtPeriod) ;
-
-/* set the directory to store tapes in. */
-void setTapeDirectory (const char *newDir) ;
-#endif
-
 /* get the name of the directory tapes are being stored in. */
 const char *getTapeDirectory (void) ;
 
@@ -110,4 +86,7 @@ void setOutputSizeLimit (long limit) ;
 
 int tapeConfigLoadCbk (void *data) ;
 
+void tapeLogGlobalStatus (FILE *fp) ;
+void tapeLogStatus (Tape tape, FILE *fp) ;
+
 #endif /* tape_h__ */
